// ATtiny85 Pi Powered Pie Partitioning and Polygon Pruning Tool
//
// Calculates the length of the chord for dividing a circle
// into even sections.  Cab also be used to create a polygon
// inside of the circle.  Resolution is 1/16 inch.
//
// Written by Boomer48: March 2021

#include "LiquidCrystal_attiny.h"

LiquidCrystal_I2C lcd(0x27, 16, 2); // set address, 16 chars, 2 lines

#define Ten_Sw 1
#define One_Sw 3
#define Frac_Sw 4
const float Sixteenth = 0.0625;

float Degrees;
float Radians;
float Diameter;
float Chord;
float Slices;
uint8_t Tens;
uint8_t Ones;
uint8_t Fracs;
char ASCII[] = "0123456789ABCDEF";
boolean Exit_Flag;
boolean First_Prompt;
boolean Restart_Flag;

//---------------------------------------------------------------------
void setup() {

  lcd.init();
  lcd.backlight();
  digitalWrite (Ten_Sw, HIGH);  // enable pull-up
  digitalWrite (One_Sw, HIGH);  // enable pull-up
  digitalWrite (Frac_Sw, HIGH);  // enable pull-up
}

void loop() {
  unsigned int temp;

  Tens = 0;
  Ones = 0;
  Fracs = 0;
  First_Prompt = true;
  Restart_Flag = false;
  lcd.clear();
  lcd.print("Pie Diameter?");
  Exit_Flag = false;

  while (Exit_Flag == false)
    Check_Switches();

  Diameter = (Tens * 10.0) + Ones + (Fracs * 0.0625);
  First_Prompt = false;
  Tens = 0;
  Ones = 0;
  Fracs = 0;
  lcd.clear();
  lcd.print("How many slices?");
  Exit_Flag = false;

  while (Exit_Flag == false)
    Check_Switches();

  Slices = (Tens * 10.0) + Ones;
  Degrees = 360.0 / Slices;
  Radians = Degrees * DEG_TO_RAD;
  Chord = Diameter * (sin(Radians / 2)) + 0.03125; // round up the fractions
  lcd.clear();
  lcd.print("Slice size =");
  temp = Chord * 16;
  Tens = temp / 160;
  temp = temp - (Tens * 160);
  Ones = temp / 16;
  Fracs = temp % 16;
  First_Prompt = true;
  Update_Display();
  delay(2000);
  Restart_Flag = true;
  Exit_Flag = false;

  // Press and hold any switch to start over
  while (Exit_Flag == false)
    Check_Switches();
}

void Update_Display() {

  lcd.setCursor(0, 1); // second line

  if (Tens > 0)
    lcd.print(ASCII[Tens]);
  else
    lcd.print(" ");

  lcd.print(ASCII[Ones]);
  lcd.print(" ");

  if (First_Prompt == true)
    switch (Fracs) {
      case 0:
        lcd.print(" ");
        lcd.print(" ");
        lcd.print(" ");
        lcd.print(" ");
        lcd.print(" ");
        break;
      case 2:
        lcd.print('1');
        lcd.print('/');
        lcd.print('8');
        lcd.print(" ");
        lcd.print(" ");
        break;
      case 4:
        lcd.print('1');
        lcd.print('/');
        lcd.print('4');
        lcd.print(" ");
        lcd.print(" ");
        break;
      case 6:
        lcd.print('3');
        lcd.print('/');
        lcd.print('8');
        lcd.print(" ");
        lcd.print(" ");
        break;
      case 8:
        lcd.print('1');
        lcd.print('/');
        lcd.print('2');
        lcd.print(" ");
        lcd.print(" ");
        break;
      case 10:
        lcd.print('5');
        lcd.print('/');
        lcd.print('8');
        lcd.print(" ");
        lcd.print(" ");
        break;
      case 12:
        lcd.print('3');
        lcd.print('/');
        lcd.print('4');
        lcd.print(" ");
        lcd.print(" ");
        break;
      case 14:
        lcd.print('7');
        lcd.print('/');
        lcd.print('8');
        lcd.print(" ");
        lcd.print(" ");
        break;
      default:
        lcd.print(Fracs);
        lcd.print('/');
        lcd.print("16");
        lcd.print(" ");
        break;
    }
}

void Check_Switches() {
  byte Sw_Num = 0;
  byte Timeout = 0;

  delayMicroseconds(16000); // debounce switch before reading
  delayMicroseconds(16000);
  delayMicroseconds(16000);

  if (digitalRead(Ten_Sw) == LOW)
    Sw_Num = 1;
  else if (digitalRead(One_Sw) == LOW)
    Sw_Num = 2;
  else if (digitalRead(Frac_Sw) == LOW)
    Sw_Num = 3;

  while ((digitalRead(Ten_Sw) == LOW) || (digitalRead(One_Sw) == LOW) || (digitalRead(Frac_Sw) == LOW))
  {
    delayMicroseconds(16000); // wait for switch release
    Timeout++;
  }

  delayMicroseconds(16000); // debounce switch release
  delayMicroseconds(16000);
  delayMicroseconds(16000);

  if (Timeout > 20)
    Exit_Flag = true; // Exit if greater than 1 second
  else if (Sw_Num > 0)
  {
    if (Sw_Num == 1)
      Tens = (Tens + 1) % 10;
    else if (Sw_Num == 2)
      Ones = (Ones + 1) % 10;
    else if (Sw_Num == 3)
      Fracs = (Fracs + 1) % 16;

    Exit_Flag = false;

    if (Restart_Flag == false)
      Update_Display();
  }
}
